<?php
/**
 *
 * MailChimp API operations
 *
 * @copyright   ArtbeesLTD (c)
 * @link        http://artbees.net
 * @since       Version 5.0
 * @last_update Version 5.0.8
 * @package     artbees
 * @author      Mucahit Yilmaz & Ugur Mirza Zeyrek
 */

defined( 'ABSPATH' ) || die();

class MK_MailChimp {

	private $api_key = '';
	private $datacenter = '';
	private $api_version = '';
	private $api_endpoint = '';
	private $format = '';
	private $verify_ssl = false;
	private $user_agent = '';
	private $debug = false;

	function __construct( $api_key ) {
		$this->api_key = $api_key;
		if ( strlen( substr( strrchr( $api_key, '-' ), 1 ) ) ) {
			$this->datacenter = substr( strrchr( $api_key, '-' ), 1 );
		} else {
			$this->datacenter = 'us1';
		}
		$this->api_version = '3.0';
		$this->api_endpoint = 'https://' . $this->datacenter . '.api.mailchimp.com/' . $this->api_version . '/';
		$this->format = 'json';
		$this->verify_ssl = false;
		$this->user_agent = 'Jupiter-Mailing/1.0';
	}

	/**
	 *
	 *  Runs API query
	 *
	 *  @param $query - string - endpoint and parameters. like "lists/subscribe"
	 *  @param $data - array - the data which'll be posted. must NOT be json decoded!
	 *  @param $data - array - optional - curl query timeout in seconds
	 *
	 *  @return array - API response
	 */
	private function rest( $query, $data = array(), $timeout = 10 ) {
		$url = $this->api_endpoint . $query;
		$key = $this->api_key;

		$response = wp_remote_post(
			$url,
			[
				'headers' => [
					'Content-Type'  => 'application/json',
					'Authorization' => 'apikey ' . $key
				],
				'body'    => json_encode( $data ),
			],
			$timeout
		);

		return json_decode( wp_remote_retrieve_body( $response ), true );
	}

	/**
	 *
	 *  Subscribes the email address to the given list
	 *
	 *  @param  $email - string
	 *  @param  $list_id - string - sth like "6edd80a499"
	 *  @param  $first_name - string - optional
	 *  @param  $last_name - string - optional
	 *  @param  $consent - boolean - optional
	 *
	 *  @return $result - array
	 */
	public function subscribe( $email, $list_id, $consent = false ) {
		$optin = jupiter_donut_get_option( 'mailchimp_optin' );
		$data  = [
			'email_address'   => $email,
			'status'          => 'subscribed',
			'double_optin'    => $optin,
			'send_welcome'    => $optin,
			'merge_fields'    => [
				'consents_to_one_to_one_messaging' => $consent,
			],
		];

		return $this->rest( 'lists/' . $list_id . '/members' , $data, 20 );
	}

	/**
	 *
	 *  Unsubscribes the email address to the given list
	 *
	 *  @param  $email - string
	 *  @param  $list_id - string - sth like "6edd80a499"
	 *  @param  $optout - boolean - optional
	 *
	 *  @return $result - array
	 */
	public function unsubscribe( $email, $list_id, $optout = false ) {
		$data = [
			'members' => [
				[
					'email_address'   => $email,
					'status'          => 'unsubscribed',
				],
			],
			'update_existing' => true,
		];

		return $this->rest( 'lists/' . $list_id , $data, 20 );
	}

	/**
	 *
	 *  Lists lists o_O
	 *
	 *  @param  $list_id - string - sth like "6edd80a499"
	 *
	 *  @return $result - array
	 */
	public function get_lists( $list_id = '' ) {
		$data = array(
			'filters' => array(
				'list_id' => $list_id,
			),
		);

		return $this->rest( 'lists/list',$data,20 );
	}
}
